@@ -277,7 +277,7 @@ class AtomDance(object):
                                    botol=self.botol)
       
       self.InitZmat = np.array(self.get_zmatrix(atoms))
-      self.write_zmat(self.InitZmat)
+      # self.write_zmat(self.InitZmat)
 
   def get_zmatrix(self,atoms):
       ''' get the zmatrix of a molecular'''
@@ -490,7 +490,6 @@ class AtomDance(object):
          sign = -1.0
       atoms = self.set_bond_momenta(i,j,atoms,sign=sign)
       return atoms
-      
 
   def bond_momenta(self,atoms):
       ratio = []
@@ -508,7 +507,6 @@ class AtomDance(object):
              return atoms,bd,groupi,groupj
       return atoms,None,[],[]
 
-
   def check_momenta(self,atoms,freeatoms=None):
       v = atoms.get_velocities()
 
@@ -540,7 +538,6 @@ class AtomDance(object):
          atoms.set_velocities(v)
       return atoms
 
-
   def set_bond_momenta(self,i,j,atoms,sign=1.0,add=False):
       ha      = int(0.5*self.natom)
       # x     = atoms.get_positions()
@@ -583,7 +580,6 @@ class AtomDance(object):
 
       atoms.set_velocities(v)
       return atoms,group_i,group_j
-      
 
   def zmat_relax(self,atoms,nbin=10,relax_step=None,
                  zmat_variable=None,zvlo=None,zvhi=None,
@@ -646,7 +642,6 @@ class AtomDance(object):
       if reset: images.append(atoms_)
       return images,relaxlog
 
-
   def get_optimal_zv(self,atoms,zmat_variable,optgen=None,nbin=100,maxiter=100,
                        plot_opt=False):
       i,j           = zmat_variable
@@ -743,7 +738,6 @@ class AtomDance(object):
          plt.close() 
       return optlog,result
 
-
   def get_dz(self,atoms,zmat,dz,bin_):
       atoms_ = atoms.copy()
       atoms_ = self.zmat_to_cartation(atoms_,zmat)
@@ -803,7 +797,6 @@ class AtomDance(object):
                     return (i,j),lo,hi
       return None,0.0,0.0
 
-
   def get_zmats(self,mdtraj='md.traj'):
       images = Trajectory(mdtraj)
       mdzmat = []
@@ -813,7 +806,6 @@ class AtomDance(object):
       mdzmat = np.array(mdzmat)
       return mdzmat
 
-
   def check_bond(self,atoms=None,mdtraj=None,rmax=1.3):
       if atoms is None:
          atoms = self.ir.atoms
@@ -860,7 +852,6 @@ class AtomDance(object):
          bB_ += 1
       return bB_,bkbd
 
-
   def checkBond(self,bonds):
       newbond = False
       bd      = None
@@ -890,7 +881,6 @@ class AtomDance(object):
       fc.close()
       return atoms
 
-
   def checkLoneAtom(self,atoms,fc):
       for i in range(self.natom):
           if self.ir.Delta[i]<=self.ir.atol:
@@ -906,7 +896,6 @@ class AtomDance(object):
              self.ir.calculate_Delta(atoms)
       return atoms
 
-
   def checkLoneAtoms(self,atoms,fc):
       for i in range(self.natom):
           if self.ir.Delta[i]<=self.ir.atol:
@@ -1220,7 +1209,6 @@ class AtomDance(object):
        
       return images
   
-
   def stretch(self,pair,atoms=None,nbin=20,rst=0.7,red=1.3,scale=1.25,traj=None,
               ToBeMoved=None,neighbors=None):
       if atoms is None:
@@ -1276,18 +1264,34 @@ class AtomDance(object):
           if not traj is None: his.write(atoms=atoms_)
       return images
 
-
-  def write_zmat(self,zmatrix,zfile=None):
+  def write_zmat(self,zmatrix,zfile=None,uspex=False):
       if zfile is None:
          zfile = self.label+'.zmat'
-      with open (zfile,'w') as f:
-           for i,iatom in enumerate(self.zmat_id):
-               i_ = self.zmat_id[i]
-               print('[ \'%s\',%3d, %3d, %3d, %3d,' %(self.atom_name[i_],iatom,self.zmat_index[i][0],
-                                              self.zmat_index[i][1],self.zmat_index[i][2]),
-                     ' %7.4f,%8.4f,%9.4f ],' %(zmatrix[i][0],zmatrix[i][1],zmatrix[i][2]),
-                     file=f)
+      
+      atom_id = [1 for i in  range(self.natom)]    # new id
+      for i,iatom in enumerate(self.zmat_id):
+          atom_id[iatom] = i
+      p = self.atoms.positions
 
+      with open (zfile,'w') as f:
+           if uspex:
+              print('MOL_{:s}'.format(self.label),file=f)
+              print('Number of atoms: {:d}'.format(self.natom),file=f)
+           for i,iatom in enumerate(self.zmat_id):
+               if uspex:
+                  st = atom_id[self.zmat_index[i][0]]+1 if self.zmat_index[i][0]>=0 else 0
+                  nd = atom_id[self.zmat_index[i][1]]+1 if self.zmat_index[i][1]>=0 else 0
+                  rd = atom_id[self.zmat_index[i][2]]+1 if self.zmat_index[i][2]>=0 else 0
+                  # print('{:4d} {:4d} {:4d} {:4d} {:4d} {:4d}'.format(st-1,nd-1,rd-1,
+                  #          self.zmat_index[i][0],self.zmat_index[i][1],self.zmat_index[i][2]))
+                  print('{:s} '.format(self.atom_name[iatom]),
+                        '{:10.6f} {:10.6f} {:10.6f} '.format(p[iatom][0],p[iatom][1],p[iatom][2]),
+                        '{:4d} {:4d} {:4d} {:4d}'.format(st,nd,rd,0),file=f)
+               else:
+                  print('[ \'%s\',%4d, %4d, %4d, %4d,' %(self.atom_name[iatom],iatom,self.zmat_index[i][0],
+                                                self.zmat_index[i][1],self.zmat_index[i][2]),
+                        ' %7.4f,%8.4f,%9.4f ],' %(zmatrix[i][0],zmatrix[i][1],zmatrix[i][2]),
+                        file=f)
 
   def close(self):
       self.ir        = None
