@@ -101,7 +101,7 @@ def SinglePointEnergies(traj,label='aimd',xcf='VDW',xca='DRSLL',basistype='DZP',
            e     = atoms_.get_potential_energy()
            E.append(e)
            E_.append(e_)
-        except AttributeError:
+        except:
            print('- current DFT SCF calculation is not converged, stopping the structure calculation ...')
            continue # break #
 
