@@ -79,7 +79,12 @@ def Basis():
                    '   1.000    1.000',
                    ' n=3   2  1',
                    '   5.0574',
-                   '   1.000']
+                   '   1.000'],
+               'Fe':['Fe  2 ',
+                     '0  2  P',
+                     '6. 0.',
+                     '2  2',
+                     '0. 0.']
                   }
     return basis
 
@@ -118,7 +123,7 @@ def single_point(atoms,id=0,xcf='VDW',xca='DRSLL',basistype='DZP',
     write_siesta_in(atoms,coord='cart',
                     md=True,opt='SinglePoint',
                     xcf=xcf,xca=xca,basistype=basistype,
-                    us=False) # for MD
+                    us=False,**kwargs) # for MD
     system('mpirun -n %d siesta<in.fdf>siesta.out' %cpu)
     
     natom     = len(atoms)
@@ -131,17 +136,17 @@ def single_point(atoms,id=0,xcf='VDW',xca='DRSLL',basistype='DZP',
     e,f,p,q = get_siesta_info(natom,spec,atom_name,val=val,label='siesta')
     system('cp siesta.out siesta-{:d}.out'.format(id))
     
-    atoms.set_initial_charges(charges=q[-1])
-    calc = SinglePointCalculator(atoms,energy=e,stress=p[-1])
+    # atoms.set_initial_charges(charges=q[-1])
+    calc = SinglePointCalculator(atoms,energy=e,forces=f[-1],stress=p[-1])
     atoms.set_calculator(calc)
+    system('rm siesta.* ')
     return atoms
 
-
 def siesta_opt(atoms=None,label='siesta',ncpu=4,supercell=[1,1,1],
                VariableCell='true',xcf='VDW',xca='DRSLL',basistype='DZP',
                dt=0.5,T=None,tstep=200,
                us='F',P=0.0,
-               gen='poscar.gen'):
+               gen='poscar.gen',**kwargs):
     if exists('siesta.MDE') or exists('siesta.MD_CAR'):
        system('rm siesta.MDE siesta.MD_CAR')
     if atoms is None:
@@ -150,15 +155,15 @@ def siesta_opt(atoms=None,label='siesta',ncpu=4,supercell=[1,1,1],
                     VariableCell=VariableCell,
                     md=False,opt='CG',P=P,tstep=tstep,
                     xcf=xcf,xca=xca,basistype=basistype,
-                    us=us) # for MD
+                    us=us,**kwargs) # for MD
     run_siesta(ncpu=ncpu)
     images = siesta_traj(label=label)
     return images
 
-
 def siesta_md(atoms=None,label='siesta',gen='poscar.gen',ncpu=4,supercell=[1,1,1],
               dt=0.5,P=0.0,T=None,tstep=5000,opt='Verlet',
-              us='F',FreeAtoms=None,xcf='VDW',xca='DRSLL',basistype='DZP'):
+              us='F',FreeAtoms=None,xcf='VDW',xca='DRSLL',basistype='DZP',
+              **kwargs):
     if exists('siesta.MDE') or exists('siesta.MD_CAR'):
        system('rm siesta.MDE siesta.MD_CAR')
     if atoms is None:
@@ -167,7 +172,7 @@ def siesta_md(atoms=None,label='siesta',gen='poscar.gen',ncpu=4,supercell=[1,1,1
                     md=True,opt=opt,
                     xcf=xcf,xca=xca,basistype=basistype,
                     dt=dt,T=T,P=P,tstep=tstep,
-                    us=us,FreeAtoms=FreeAtoms) # for MD
+                    us=us,FreeAtoms=FreeAtoms,**kwargs) # for MD
     run_siesta(ncpu=ncpu)
     images = siesta_traj(label=label,batch=tstep)
     return images
@@ -176,7 +181,6 @@ def siesta_md(atoms=None,label='siesta',gen='poscar.gen',ncpu=4,supercell=[1,1,1
 def run_siesta(ncpu=12):
     system('mpirun -n %d siesta<in.fdf>siesta.out' %ncpu)
 
-
 def write_siesta_in(Atoms,coord='zmatrix',
                     VariableCell='false',
                     fac=1.0,
@@ -187,10 +191,10 @@ def write_siesta_in(Atoms,coord='zmatrix',
                     basistype='DZP',
                     spin='F',
                     dt=0.5,T=300,P=0.0,tstep=3000,
-                    us='F',
-                    maxiter=500,
+                    us='F', # MaxSCFIterations=200,
                     singlek=False,
-                    FreeAtoms=None):
+                    FreeAtoms=None,
+                    **kwargs):
     ''' siesta input generation
         us = use saved data
     ''' 
@@ -241,9 +245,12 @@ def write_siesta_in(Atoms,coord='zmatrix',
     print('   ',file=fdf)
     print('   ',file=fdf)
     print('SolutionMethod   Diagon # ## OrderN or Diagon',file=fdf)
-    print('MaxSCFIterations %d' %maxiter,file=fdf)
+    # print('MaxSCFIterations %d' %MaxSCFIterations,file=fdf)
+    
+    if 'MaxSCFIterations' in kwargs:
+       print('MaxSCFIterations        {:d} \n'.format(kwargs['MaxSCFIterations']),file=fdf)
     # print('PAO.EnergyShift  100 meV',file=fdf)
-
+    
     if basistype=='split':
        print('PAO.BasisType    split',file=fdf)
        print('%block PAO.Basis',file=fdf)
@@ -257,7 +264,13 @@ def write_siesta_in(Atoms,coord='zmatrix',
     print('SpinPolarized    %s' %spin,file=fdf)
     print('   ',file=fdf)
     print('   ',file=fdf)
-    print('DM.MixingWeight      0.4   ',file=fdf)
+
+    if 'InitSpinAF' in kwargs:
+       print('DM.InitSpinAF        {:s} \n'.format(kwargs['InitSpinAF']),file=fdf)
+    if 'MixingWeight' in kwargs:
+       print('DM.MixingWeight      {:f} \n'.format(kwargs['MixingWeight']),file=fdf)
+    else:
+       print('DM.MixingWeight      0.4   ',file=fdf)
     print('DM.NumberPulay       9',file=fdf)
     print('DM.Tolerance         1.d-4',file=fdf)
     print('   ',file=fdf)
@@ -296,7 +309,10 @@ def write_siesta_in(Atoms,coord='zmatrix',
     print('XC.Authors       %s    # Particular parametrization of xc func'%xca,file=fdf)
     print('   ',file=fdf)
     print('   ',file=fdf)
-    print('MeshCutoff       200. Ry # Equivalent planewave cutoff for the grid ',file=fdf)
+    if 'MeshCutoff' in kwargs:
+       print('MeshCutoff        {:f} Ry \n'.format(kwargs['MeshCutoff']),file=fdf)
+    else:
+       print('MeshCutoff       200. Ry # Equivalent planewave cutoff for the grid ',file=fdf)
     print('KgridCutoff      %f Ang' %kgrid,file=fdf)
     print('   ',file=fdf)
     print('   ',file=fdf)
@@ -396,7 +412,6 @@ def write_siesta_in(Atoms,coord='zmatrix',
        print('    ',file=fdf)
     fdf.close()
 
-
 def print_zmatrix(ind,pos,neighors,nneigh,species,atoms_label,table,fdf):
     # print(atoms_label[ind],ind,table[ind])
     if not ind in neighors:
@@ -569,7 +584,6 @@ def pseudo_gen(pa='vw',rc_dic=None):
         fp.close()
         print('* generating pseudo for %s, by Author %s    ' %(key,pa))
         system('./pg.sh '+key+'.'+pa+'.inp' )
-        
 
 def get_siesta_energy(label='siesta'):
     fe = open(label+'.MDE','r')
@@ -581,7 +595,6 @@ def get_siesta_energy(label='siesta'):
     nframe = int(l[0])
     return nframe
 
-
 def get_siesta_info(natom,spec,atom_name,val,label='siesta'):
     fo = open(label+'.out','r') 
     lines= fo.readlines()
@@ -604,10 +617,7 @@ def get_siesta_info(natom,spec,atom_name,val,label='siesta'):
 
     iframe = 0
     spec_  = []
-    force   = []
-    press   = []
     pressure= []
-
     e = 0.0
 
     for i,line in enumerate(lines):
@@ -615,6 +625,7 @@ def get_siesta_info(natom,spec,atom_name,val,label='siesta'):
            e = float(line.split()[-1])
         elif line.find('Atomic forces (eV/Ang)')>=0:
            ln = lines[i+1]
+           force   = []
            if ln.find('----------------------------------------')<0:
               for na in range(natom):
                  fl = lines[na+i+1] #.split()
@@ -626,6 +637,7 @@ def get_siesta_info(natom,spec,atom_name,val,label='siesta'):
            if len(force)>0:
               forces.append(force)
         elif line.find('Stress tensor (static)')>=0:
+            press   = []
             for l in range(3):
                 pl = lines[l+i+1].split()
                 press.append([float(pl[1]),float(pl[2]),float(pl[3])])
@@ -633,91 +645,7 @@ def get_siesta_info(natom,spec,atom_name,val,label='siesta'):
         elif line.find('Pressure (static):')>=0:
             l = lines[i+4].split()
             pressure.append(float(l[2]))
-        elif line.find('mulliken: Atomic and Orbital Populations:')>=0:
-           # print('-  current frame %d, MD step %d...' %(iframe,frame))
-           if iframe==0:
-              cl    = 0
-              end_  = True
-              while end_:
-                    cl   += 1
-                    line_ = lines[i+cl]
-                    if line_.find('mulliken: Qtot')>=0:
-                       end_ = False
-
-                    if line_.find('Species:')>=0:
-                       sl = 0
-                       spec_.append(line_.split()[1])
-                       
-                       qline_ = lines[i+cl+1]
-                       if qline_.find('Atom  Qatom  Qorb')<0:
-                          print('-  an error case ... ... ')
-
-                       qline_ = lines[i+cl+2]
-                       ql_    = qline_.split()
-                       nob    = len(ql_)
-
-                       o      = 0
-                       spec_end = True
-                       while spec_end:
-                             o += 1
-                             qline_ = lines[i+cl+2+o]
-                             ql_    = qline_.split()
-                             if len(ql_)== nob+2:
-                                obs[spec_[-1]] = o
-                                spec_end = False
-
-              # print('\n Qorb: \n',obs)
-              q_    = np.zeros([natom])
-              cl    = 0
-              end_  = True
-              while end_:
-                    cl   += 1
-                    line_ = lines[i+cl]
-                    if line_.find('mulliken: Qtot')>=0:
-                       end_ = False
-
-                    if line_.find('Species:')>=0:
-                       sl = 0
-                       s_ = line_.split()[1]
-                       # print('\n-  charges of species: %s \n' %s_)
-                       
-                       for i_ in range(nsp[s_]):
-                           qline_ = lines[i+cl+2+(i_+1)*obs[s_]]
-                           ql_    = qline_.split()
-
-                           ai     = int(ql_[0])-1
-                           q_[ai] = val[s_] - float(ql_[1])  # original is oppose!
-
-                       cl += 2+i_*obs[s_]
-              qs.append(q_)
-           else:
-              q_    = np.zeros([natom])
-              cl    = 0
-              end_  = True
-              while end_:
-                    cl   += 1
-                    line_ = lines[i+cl]
-                    if line_.find('mulliken: Qtot')>=0:
-                       end_ = False
-
-                    if line_.find('Species:')>=0:
-                       sl = 0
-                       s_ = line_.split()[1]
-
-                       for i_ in range(nsp[s_]):
-                           qline_ = lines[i+cl+2+(i_+1)*obs[s_]]
-                           ql_    = qline_.split()
-                           
-                           ai     = int(ql_[0])-1
-                           q_[ai] = float(ql_[1])-val[s_]
-
-                       cl += 2+i_*obs[s_]
-              # print('-  frame: %d' %iframe,q_)
-              qs.append(q_)
-
-           iframe += 1
-    return e,np.array(forces),np.array(presses),np.array(qs)
-
+    return e,np.array(forces),np.array(presses),np.zeros(natom) # np.array(qs)
 
 def get_siesta_cart(label='siesta'):
     nframe = get_siesta_energy()
@@ -748,7 +676,6 @@ def get_siesta_cart(label='siesta'):
               if l[1]=='AtomicCoordinatesAndAtomicSpecies':
                  atml= i+1
 
-
     sp = []
     for isp in range(ns):
         l = lines[spl+isp].split() 
@@ -805,7 +732,6 @@ def get_siesta_cart(label='siesta'):
     cell=np.array([a,b,c])
     his.close()
 
-
 def get_siesta_fdf(label='siesta'):
     fin = open(label+'.fdf','r') 
     lines= fin.readlines()
@@ -825,7 +751,6 @@ def get_siesta_fdf(label='siesta'):
         if ns and natom:
            break
 
-
     spec      = []
     positions = []
     atom_name = []
