@@ -92,7 +92,8 @@ class IRFF_NP(object):
 
       if libfile.endswith('.json'):
          lf = open(libfile,'r')
-         j = js.load(lf)
+         j                   = js.load(lf)
+         #self.j             = j
          self.p              = j['p']
          m                   = j['m']
          self.MolEnergy_     = j['MolEnergy']
@@ -524,9 +525,11 @@ class IRFF_NP(object):
          self.ppeng = np.multiply(self.P['Depp'],self.bopp)
          self.esi   = self.sieng + self.pieng + self.ppeng
       elif self.EnergyFunction==1: 
-         self.esi = self.f_nn('fe',[self.bosi,self.bopi,self.bopp],layer=self.be_layer[1])  
+         esi      = self.f_nn('fe',[self.bosi,self.bopi,self.bopp],layer=self.be_layer[1])
+         self.esi = esi*np.where(self.bo0<0.0000001,0.0,1.0)
       elif self.EnergyFunction==2:
          self.esi = self.f_nn('fe',[-self.bosi,-self.bopi,-self.bopp],layer=self.be_layer[1]) 
+         self.esi = self.esi*np.where(self.bo0<0.0000001,0.0,1.0)
       elif self.EnergyFunction==3: 
          e_ = self.f_nn('fe',[self.bosi,self.bopi,self.bopp],layer=self.be_layer[1])  
          self.esi = self.bo0*e_
@@ -666,7 +669,6 @@ class IRFF_NP(object):
       thet_      = 180.0 - self.P['theta0']*(1.0-np.exp(-self.P['val10']*(2.0-self.SBO3)))
       self.thet0 = thet_/57.29577951
 
-
   def get_eangle(self):
       self.Dang  = self.Delta - self.P['valang']
       self.boaij = np.squeeze(self.bo[self.angi,self.angj])
@@ -694,7 +696,6 @@ class IRFF_NP(object):
       self.get_epenalty(self.boaij,self.boajk)
       self.get_three_conj(self.boaij,self.boajk)
 
-
   def f7(self,boij,bojk):
       self.expaij = np.exp(-self.P['val3']*np.power(boij+self.safety_value,self.P['val4']))
       self.expajk = np.exp(-self.P['val3']*np.power(bojk+self.safety_value,self.P['val4']))
@@ -702,13 +703,11 @@ class IRFF_NP(object):
       fk          = 1.0 - self.expajk
       self.f_7    = fi*fk
 
-
   def f8(self,dang):
       exp6      = np.exp( self.P['val6']*dang)
       exp7      = np.exp(-self.P['val7']*dang)
       self.f_8  = self.P['val5'] - (self.P['val5'] - 1.0)*(2.0+exp6)/(1.0+exp6+exp7)
 
-
   def get_epenalty(self,boij,bojk):
       self.f9()
       expi      = np.exp(-self.P['pen2']*np.square(boij-2.0))
@@ -716,14 +715,12 @@ class IRFF_NP(object):
       self.epen = self.P['pen1']*self.f_9*expi*expk*self.fijk
       self.Epen = np.sum(self.epen)
 
-
   def f9(self):
       D         = np.squeeze(self.Dv[self.angj])
       exp3      = np.exp(-self.P['pen3']*D)
       exp4      = np.exp( self.P['pen4']*D)
       self.f_9  = np.divide(2.0+exp3,1.0+exp3+exp4)
 
-
   def get_three_conj(self,boij,bojk):
       Dcoa_ = self.Delta-self.P['valboc']
       Dcoa  = Dcoa_[self.angj]
@@ -740,7 +737,6 @@ class IRFF_NP(object):
       # print(texp0.shape)
       # print(self.etcon.shape)
       self.Etcon = np.sum(self.etcon)
-  
 
   def get_torsion_angle(self):
       rij = self.r[self.tori,self.torj]
@@ -797,7 +793,6 @@ class IRFF_NP(object):
       self.w= np.arccos(self.cos_w)
       self.cos2w = np.cos(2.0*self.w)
 
-
   def get_etorsion(self):
       self.get_torsion_angle()
 
@@ -828,21 +823,18 @@ class IRFF_NP(object):
       # print('Etor',self.etor.shape)
       self.get_four_conj(self.botij,self.botjk,self.botkl)
 
-
   def f10(self,boij,bojk,bokl):
       exp1 = 1.0 - np.exp(-self.P['tor2']*boij)
       exp2 = 1.0 - np.exp(-self.P['tor2']*bojk)
       exp3 = 1.0 - np.exp(-self.P['tor2']*bokl)
       self.f_10 = exp1*exp2*exp3
 
-
   def f11(self,Dj,Dk):
       delt      = Dj+Dk
       f11exp3   = np.exp(-self.P['tor3']*delt)
       f11exp4   = np.exp( self.P['tor4']*delt)
       self.f_11 = np.divide(2.0+f11exp3,1.0+f11exp3+f11exp4)
 
-
   def get_four_conj(self,boij,bojk,bokl):
       exptol= np.exp(-self.P['cot2']*np.square(self.atol - 1.5))
       expij = np.exp(-self.P['cot2']*np.square(boij-1.5))-exptol
@@ -855,13 +847,11 @@ class IRFF_NP(object):
       # print('efcon',self.efcon.shape)
       self.Efcon = np.sum(self.efcon)
 
-
   def f13(self,r):
       rr = np.power(r,self.P['vdw1'])+np.power(np.divide(1.0,self.P['gammaw']),self.P['vdw1'])
       f_13 = np.power(rr,np.divide(1.0,self.P['vdw1']))  
       return f_13
 
-
   def get_tap(self,r):
       tpc = 1.0+np.divide(-35.0,np.power(self.vdwcut,4.0))*np.power(r,4.0)+ \
             np.divide(84.0,np.power(self.vdwcut,5.0))*np.power(r,5.0)+ \
@@ -881,7 +871,6 @@ class IRFF_NP(object):
          tp = tpc
       return tp,tpc
 
-
   def get_evdw(self,cell):
       self.evdw = 0.0
       self.ecoul= 0.0
@@ -918,7 +907,6 @@ class IRFF_NP(object):
       self.Evdw  = np.sum(self.evdw)
       self.Ecoul = np.sum(self.ecoul)
 
-  
   def get_ehb(self,cell):
       self.BOhb   = self.bo0[self.hbi,self.hbj] # np.squeeze(self.bo0[self.hbi,self.hbj])
       fhb         = self.fhb[self.hbi,self.hbj] # np.squeeze(self.fhb[self.hbi,self.hbj])
@@ -954,7 +942,6 @@ class IRFF_NP(object):
                   ehb    = fhb*frhb*self.P['Dehb']*exphb1*exphb2*sin4 
                   self.Ehb += np.sum(ehb)
 
-
   def get_eself(self):
       chi    = np.expand_dims(self.P['chi'],axis=0)
       mu     = np.expand_dims(self.P['mu'],axis=0)
@@ -962,7 +949,6 @@ class IRFF_NP(object):
       # print(self.eself.shape)
       self.Eself = np.sum(self.eself)
 
-
   def get_total_energy(self,cell,rcell,positions):
       self.get_ebond(cell,rcell,positions)
       self.get_elone()
@@ -997,7 +983,6 @@ class IRFF_NP(object):
                self.Ehb + self.Eself + self.zpe
       return E
 
-
   def update_p(self,libfile='ffield.json'):
       if libfile.endswith('.json'):
          lf = open(libfile,'r')
@@ -1260,7 +1245,10 @@ class IRFF_NP(object):
           unit_ = self.unit if key in self.punit else 1.0
           for t in self.Tors:
               pn = key + '_' + t
-              self.p[pn] = self.p[pn]*unit_
+              if pn in self.p:
+                 self.p[pn] = self.p[pn]*unit_
+              else:
+                 self.p[pn] = 0.0
 
       for h in self.Hbs:
           pn = 'Dehb_' + h
@@ -1430,15 +1418,30 @@ class IRFF_NP(object):
                  tor4 = 'X'+'-'+t2+'-'+t3+'-'+'X'
                  tor5 = 'X'+'-'+t3+'-'+t2+'-'+'X'
                  if tor1 in self.torp:
-                    self.p[key+'_'+tor] = self.p[key+'_'+tor1]
+                    try:
+                       self.p[key+'_'+tor] = self.p[key+'_'+tor1]
+                    except:
+                       self.p[key+'_'+tor] = 0.0
                  elif tor2 in self.torp:
-                    self.p[key+'_'+tor] = self.p[key+'_'+tor2]
+                    try:
+                       self.p[key+'_'+tor] = self.p[key+'_'+tor2]
+                    except:
+                       self.p[key+'_'+tor] = 0.0
                  elif tor3 in self.torp:
-                    self.p[key+'_'+tor] = self.p[key+'_'+tor3]    
+                    try:
+                       self.p[key+'_'+tor] = self.p[key+'_'+tor3]
+                    except:
+                       self.p[key+'_'+tor] = 0.0  
                  elif tor4 in self.torp:
-                    self.p[key+'_'+tor] = self.p[key+'_'+tor4]  
+                    try:
+                       self.p[key+'_'+tor] = self.p[key+'_'+tor4]
+                    except:
+                       self.p[key+'_'+tor] = 0.0
                  elif tor5 in self.torp:
-                    self.p[key+'_'+tor] = self.p[key+'_'+tor5]     
+                    try:
+                       self.p[key+'_'+tor] = self.p[key+'_'+tor5]
+                    except:
+                       self.p[key+'_'+tor] = 0.0   
                  else:
                     self.p[key+'_'+tor] = 0.0
       
