@@ -36,7 +36,7 @@ def write_gulp_in(A,runword='gradient qiterative nosymmetry conv verb debu',
                   maxcyc=2000,
                   output=None,
                   restart=100,
-                  lib='reax',movieFrequency=10):
+                  lib='reax',movieFrequency=10,**kwargs):
     ''' runword = keyword in gulp input
         can be 'md conv' 'md conp' 'opti conp'
         qite = iterative solve charges, norx = no charges '''
@@ -47,16 +47,24 @@ def write_gulp_in(A,runword='gradient qiterative nosymmetry conv verb debu',
     print('#',file=finp)
     if rw[0]=='opti':
        print('maxcyc %d' %maxcyc,file=finp)
+       if 'gtol' in kwargs:
+          print('gtol opt  {:f} '.format(kwargs['gtol']),file=finp)
+       else:
+          print('gtol opt  0.00001',file=finp)
     elif rw[0]=='md':
-       if rw[1]=='conv':
-          print('ensemble nvt',file=finp)
-          print('tau_thermostat     0.05 ps',file=finp)
-       elif rw[1]=='conp':
-          print('integrator leapfrog verlet',file=finp)
-          print('ensemble           npt',file=finp)
-          print('pressure           0.00 GPa',file=finp)
-          print('tau_barostat       0.1 ps',file=finp)
-          print('tau_thermostat     0.1 ps',file=finp)
+       #  if rw[1]=='conv':
+       #     print('ensemble nvt',file=finp)
+       #     print('tau_thermostat     0.05 ps',file=finp)
+       #  if rw[1]=='conp':
+       #     print('integrator leapfrog verlet',file=finp)
+       #     print('ensemble           npt',file=finp)
+       #     print('pressure           0.00 GPa',file=finp)
+       #     print('tau_barostat       0.1 ps',file=finp)
+       #     print('tau_thermostat     0.1 ps',file=finp)
+       if 'ensemble' in kwargs:
+          print('ensemble   =  {:s} '.format(kwargs['ensemble']),file=finp)
+       if 'tau_thermostat' in kwargs:
+          print('tau_thermostat   =  {:f} ps'.format(kwargs['tau_thermostat']),file=finp)
 
        print('temperature        %f K' %T,file=finp)
        print('timestep           %f fs' %time_step,file=finp)
@@ -79,6 +87,7 @@ def write_gulp_in(A,runword='gradient qiterative nosymmetry conv verb debu',
        print('output movie {:d} xyz his.xyz'.format(movieFrequency),file=finp)
     else:
        print('output {:s}'.format(output),file=finp)
+    print('output cif gulp produces gulp.cif',file=finp)
     print('dump {:d} restart.grs'.format(restart),file=finp)
     print('#',file=finp)
     
@@ -284,9 +293,9 @@ def reaxyz(fxyz):
                positions[nf][na][2] = float(l[3])
     return atom_name,positions,energies
 
-def xyztotraj(fxyz,checkMol=False,mode='w',scale=True):
+def xyztotraj(fxyz,inp='inp-gulp',checkMol=False,mode='w',scale=True):
     atom_name,positions,e = reaxyz(fxyz)
-    cell = get_lattice()
+    cell = get_lattice(inp=inp)
     u    = np.linalg.inv(cell)
     his  = TrajectoryWriter('gulp.traj',mode=mode)
 
@@ -435,31 +444,31 @@ def init_bonds(p_):
            hbs.append(k[1])
     return spec,bonds,offd,angs,torp,hbs
 
-def get_gulp_energy(atoms,ffild='reaxff_nn'):
+def get_gulp_energy(atoms,ffield='reaxff_nn'):
     write_gulp_in(atoms,runword='gradient nosymmetry conv qite verb',
                   lib=ffield)
     system('gulp<inp-gulp>out')
 
     (e_,eb_,el_,eo_,eu_,ea_,ep_,
     etc_,et_,ef_,ev_,ehb_,ecl_,esl_)= get_reax_energy(fo='out')
-    GE = {}
-    GE['ebond']        = eb_
-    GE['elonepair']    = el_
-    GE['eover']        = eo_
-    GE['eunder']       = eu_
-    GE['eangle']       = ea_
-    GE['econjugation'] = etc_
-    GE['epenalty']     = ep_
-    GE['etorsion']     = et_
-    GE['fconj']        = ef_
-    GE['evdw']         = ev_
-    GE['ecoul']        = ecl_
-    GE['ehb']          = ehb_
-    GE['eself']        = esl_
-    GE['E']            = e_
-    return GE
+    ge = {}
+    ge['ebond']        = eb_
+    ge['elonepair']    = el_
+    ge['eover']        = eo_
+    ge['eunder']       = eu_
+    ge['eangle']       = ea_
+    ge['econjugation'] = etc_
+    ge['epenalty']     = ep_
+    ge['etorsion']     = et_
+    ge['fconj']        = ef_
+    ge['evdw']         = ev_
+    ge['ecoul']        = ecl_
+    ge['ehb']          = ehb_
+    ge['eself']        = esl_
+    ge['E']            = e_
+    return ge
 
-def get_gulp_force_from_drv(fdrv):
+def get_gulp_force_from_drv(fdrv,wdftb=False):
     forces = []
     with open(fdrv,'r') as fdrv:
          gl = 100
@@ -470,17 +479,22 @@ def get_gulp_force_from_drv(fdrv):
              if i>gl and i<=gl+natom:
                 forces.append([float(line.split()[1]),float(line.split()[2]),float(line.split()[3])])
     # print(forces)
+    if wdftb:
+       with open('results.tag','w') as fdftb:
+            print('forces              :real:2:',file=fdftb)
+            for f in forces:
+               print(f[0],f[1],f[2],file=fdftb)
     return np.array(forces)
 
-def get_gulp_forces(images,traj='gulp_force.traj',ffield='reaxff_nn'):
+def get_gulp_forces(images,traj='gulp_force.traj',ffield='reaxff_nn',wdftb=False):
     ''' calculate the force by GULP '''
     his  = TrajectoryWriter(traj,mode='w')
-    for i,atoms in enumerate(images):
+    for atoms in images:
         write_gulp_in(atoms,runword='gradient nosymmetry conv qite verb',
                       output='drv gulp',restart=1,
                       lib=ffield)
         system('gulp<inp-gulp>out')
-        forces = get_gulp_force_from_drv('gulp.drv')
+        forces = get_gulp_force_from_drv('gulp.drv',wdftb=wdftb)
         #atoms.forces = forces
         (e,eb_,el_,eo_,eu_,ea_,ep_,
          etc_,et_,ef_,ev_,ehb_,ecl_,esl_)= get_reax_energy(fo='out')
@@ -491,3 +505,4 @@ def get_gulp_forces(images,traj='gulp_force.traj',ffield='reaxff_nn'):
         atoms.set_calculator(calc)
         his.write(atoms=atoms)
     his.close()
+    return atoms
