@@ -49,13 +49,13 @@ class MPNN(ReaxFF):
                batch_size=200,sample='uniform',
                hbshort=6.75,hblong=7.5,
                vdwcut=10.0,
-               beup={},                                 # e.g. {'C-C':[(1.5,0.5)]} or {'C-C':[(1.5,'si')]}
+               beup={},                          # e.g. {'C-C':[(1.5,0.5)]} or {'C-C':[(1.5,'si')]}
                belo={},
                vlo={'others':[(0.0,0.0)]},
                vup={'others':[(10.0,0.0)]},
                pim={'others':10.0},
                spv_be=False,                             
-               spv_bo=None,                             # e.g. spv_bo={'C-C':(3.0,0.2,1.0)}
+               spv_bo=None,                      # e.g. spv_bo={'C-C':(3.0,0.2,1.0)}
                spv_pi=False,
                spv_ang=False,
                spv_vdw=False,
@@ -164,6 +164,8 @@ class MPNN(ReaxFF):
       self.S        = {}
       self.esi      = {}
       self.fbo      = {}
+      self.Dbi      = {}
+      self.Dbj      = {}
 
   def get_loss(self):
       self.Loss = 0.0
@@ -393,7 +395,6 @@ class MPNN(ReaxFF):
          self.Dp     = tf.gather_nd(self.BOP,self.dlist)  
          self.Deltap = tf.reduce_sum(input_tensor=self.Dp,axis=1,name='Deltap')
 
-
   def get_bondorder(self,t,bd,atomi,atomj):
       ''' compute bond-order according the message function '''
       Di      = tf.gather_nd(self.D[t-1],self.dilink[bd])
@@ -427,11 +428,11 @@ class MPNN(ReaxFF):
          bopi = self.Hpi[t-1][bd]*Fpi
          bopp = self.Hpp[t-1][bd]*Fpp
       elif self.MessageFunction==2:
-         Dbi  = Di-h
-         Dbj  = Dj-h
-         Fi   = self.fmessage(flabel,b[0],self.nbd[bd],[Dbi,Dbj,self.Hsi[t-1][bd],self.Hpi[t-1][bd],self.Hpp[t-1][bd]],
+         self.Dbi[bd]  = Di-h
+         self.Dbj[bd]  = Dj-h
+         Fi   = self.fmessage(flabel,b[0],self.nbd[bd],[self.Dbi[bd],self.Dbj[bd],self.Hsi[t-1][bd],self.Hpi[t-1][bd],self.Hpp[t-1][bd]],
                               layer=self.mf_layer[1])
-         Fj   = self.fmessage(flabel,b[1],self.nbd[bd],[Dbj,Dbi,self.Hsi[t-1][bd],self.Hpi[t-1][bd],self.Hpp[t-1][bd]],
+         Fj   = self.fmessage(flabel,b[1],self.nbd[bd],[self.Dbj[bd],self.Dbi[bd],self.Hsi[t-1][bd],self.Hpi[t-1][bd],self.Hpp[t-1][bd]],
                               layer=self.mf_layer[1])
          F    = Fi*Fj
          Fsi,Fpi,Fpp = tf.unstack(F,axis=2)
@@ -440,10 +441,10 @@ class MPNN(ReaxFF):
          bopi = self.Hpi[t-1][bd]*Fpi
          bopp = self.Hpp[t-1][bd]*Fpp
       elif self.MessageFunction==3:
-         Dbi  = Di - h # self.p['valboc_'+atomi]  
-         Dbj  = Dj - h # self.p['valboc_'+atomj]  
-         Fi   = self.fmessage(flabel,b[0],self.nbd[bd],[Dbi,h,Dbj],layer=self.mf_layer[1])
-         Fj   = self.fmessage(flabel,b[1],self.nbd[bd],[Dbj,h,Dbi],layer=self.mf_layer[1])
+         self.Dbi[bd]  = Di - h # self.p['valboc_'+atomi]  
+         self.Dbj[bd]  = Dj - h # self.p['valboc_'+atomj]  
+         Fi   = self.fmessage(flabel,b[0],self.nbd[bd],[self.Dbi[bd],h,self.Dbj[bd]],layer=self.mf_layer[1])
+         Fj   = self.fmessage(flabel,b[1],self.nbd[bd],[self.Dbj[bd],h,self.Dbi[bd]],layer=self.mf_layer[1])
          F    = Fi*Fj
          Fsi,Fpi,Fpp = tf.unstack(F,axis=2)
 
@@ -455,10 +456,10 @@ class MPNN(ReaxFF):
          # Dbj  = Dj - self.p['val_'+atomj]
          # f_1  = self.f1(bd,atomi,atomj,Di,Dj,Dbi,Dbj) # over correction 
 
-         Di_boc = Di - self.p['val_'+atomi]        # bo correction
-         Dj_boc = Di - self.p['val_'+atomi]
-         Fi   = self.f_nn(flabel,b[0],self.nbd[bd],[Di_boc,h,Dj_boc],layer=self.mf_layer[1])
-         Fj   = self.f_nn(flabel,b[1],self.nbd[bd],[Dj_boc,h,Di_boc],layer=self.mf_layer[1])
+         self.Dbi[bd]  = Di - self.p['val_'+atomi]        # bo correction
+         self.Dbj[bd] = Di - self.p['val_'+atomi]
+         Fi   = self.f_nn(flabel,b[0],self.nbd[bd],[self.Dbi[bd],h,self.Dbj[bd]],layer=self.mf_layer[1])
+         Fj   = self.f_nn(flabel,b[1],self.nbd[bd],[self.Dbj[bd],h,self.Dbi[bd]],layer=self.mf_layer[1])
 
          #one  = lambda:1.0
          #F1   = lambda:f_1*f_1
@@ -470,10 +471,10 @@ class MPNN(ReaxFF):
          bopi = self.Hpi[t-1][bd]*F
          bopp = self.Hpp[t-1][bd]*F 
       elif self.MessageFunction==5:
-         Dbi  = Di - self.p['val_'+atomi] # Di-h
-         Dbj  = Dj - self.p['val_'+atomj] # Dj-h
-         Fi   = self.fmessage(flabel,b[0],self.nbd[bd],[Dbi,h,Dbj],layer=self.mf_layer[1])
-         Fj   = self.fmessage(flabel,b[1],self.nbd[bd],[Dbj,h,Dbi],layer=self.mf_layer[1])
+         self.Dbi[bd]  = Di - self.p['val_'+atomi] # Di-h
+         self.Dbj[bd]  = Dj - self.p['val_'+atomj] # Dj-h
+         Fi   = self.fmessage(flabel,b[0],self.nbd[bd],[self.Dbi[bd],h,self.Dbj[bd]],layer=self.mf_layer[1])
+         Fj   = self.fmessage(flabel,b[1],self.nbd[bd],[self.Dbj[bd],h,self.Dbi[bd]],layer=self.mf_layer[1])
          F    = Fi*Fj
          Fsi,Fpi,Fpp = tf.unstack(F,axis=2)
 
@@ -964,9 +965,6 @@ class MPNN(ReaxFF):
       self.MolEnergy_ = self.sess.run(self.MolEnergy)
       for key in self.MolEnergy_:
           self.MolEnergy_[key] = float(self.MolEnergy_[key])
-          
-      if not loss is None:
-         self.p_['score'] = -loss
 
       for k in p_:
           key = k.split('_')[0]
@@ -974,11 +972,12 @@ class MPNN(ReaxFF):
              k_ = k.split('_')[1]
              if k_ not in self.torp:
                 continue
-
           self.p_[k] = float(p_[k])
           if key in self.punit:
              self.p_[k] = float(p_[k]/self.unit)
 
+      score = loss if loss is None else -loss
+         
       if self.libfile.endswith('.json'):
          self.m_   = self.sess.run(self.m)
 
@@ -993,6 +992,7 @@ class MPNN(ReaxFF):
          # print(' * save parameters to file ...')
          fj = open(libfile+'.json','w')
          j = {'p':self.p_,'m':self.m_,
+              'score':score,
               'BOFunction':self.BOFunction,
               'EnergyFunction':self.EnergyFunction,
               'MessageFunction':self.MessageFunction, 
@@ -1091,62 +1091,37 @@ class MPNN(ReaxFF):
              penalty  = tf.add(self.penalty_bop[bd]*self.lambda_bd,penalty)
 
              if self.spv_be:
-                # if self.MessageFunction==1:
-                # fe  = tf.where(tf.less_equal(self.rbd[bd],self.rc_bo[bd]),1.0,0.0) ##### r< r_e that bo > bore_
-                # else:
-                #   fe  = tf.where(tf.less_equal(self.rbd[bd],self.rc_bo[bd]),1.0,0.0) 
-                self.penalty_be[bd] = tf.constant(0.0)
-                if (bd in self.beup) or (bdr in self.beup):
-                   bd_ = bd if bd in self.beup else bdr
-                   for beup_ in self.beup[bd_]:
-                       r_,be_u = beup_  
-                       if isinstance(be_u,str):
-                          if be_u=='si':
-                             be = self.bop_si[bd]
-                          elif be_u=='pi':
-                             be = self.bop_pi[bd]
-                          elif be_u=='pp':
-                             be = self.bop_pp[bd]
-                          else:
-                             raise NotImplementedError('-  This format not supported yet!')
-                          fu      = tf.where(tf.greater(self.rbd[bd],r_),1.0,0.0) ##### 
-                          fluct_u = 1.0+self.fluctuation
-                          pen_e = tf.reduce_sum(input_tensor=tf.nn.relu((self.esi[bd] - be*fluct_u)*fu))
-                       else:
-                         fu      = tf.where(tf.less_equal(self.rbd[bd],r_),1.0,0.0) ##### 
-                         pen_e   = tf.reduce_sum(input_tensor=tf.nn.relu((self.EBD[bd] - be_u)*fu))
-                       self.penalty_be[bd] = self.penalty_be[bd] + pen_e
-                       
-                if (bd in self.belo) or (bdr in self.belo):
-                   bd_ = bd if bd in self.belo else bdr
-                   for belo_ in self.belo[bd_]:
-                       r_,be_l = belo_
-                       
-                       if isinstance(be_l,str):
-                          if be_l=='si':
-                             be = self.bop_si[bd]
-                          elif be_l=='pi':
-                             be = self.bop_pi[bd]
-                          elif be_l=='pp':
-                             be = self.bop_pp[bd]
-                          else:
-                             raise NotImplementedError('-  This format not supported yet!')
-                          fl      = tf.where(tf.less_equal(self.rbd[bd],r_),1.0,0.0)   #####
-                          fluct_l = 1.0-self.fluctuation
-                          pen_e = tf.reduce_sum(input_tensor=tf.nn.relu((be*fluct_l-self.esi[bd])*fl))
-                       else:
-                          fl      = tf.where(tf.greater(self.rbd[bd],r_),1.0,0.0)   #####
+                if self.MessageFunction>1:
+                   self.penalty_be[bd] = tf.constant(0.0)
+                   if (bd in self.beup) or (bdr in self.beup):
+                      bd_ = bd if bd in self.beup else bdr
+                      for beup_ in self.beup[bd_]:
+                          r_,Di_l,Di_u,Dj_l,Dj_u,be_u = beup_  
+                          fu = tf.where(tf.logical_and(tf.less_equal(self.rbd[bd],r_),
+                                             tf.logical_and(
+                                                 tf.logical_and(tf.less_equal(self.Dbi[bd],Di_u),
+                                                                tf.greater_equal(self.Dbi[bd],Di_l)),
+                                                 tf.logical_and(tf.less_equal(self.Dbj[bd],Dj_u),
+                                                                tf.greater_equal(self.Dbj[bd],Dj_l))
+                                                         ) ), 1.0,0.0) ##### 
+                          pen_e   = tf.reduce_sum(input_tensor=tf.nn.relu((self.EBD[bd] - be_u)*fu))
+                          self.penalty_be[bd] = self.penalty_be[bd] + pen_e
+                        
+                   if (bd in self.belo) or (bdr in self.belo):
+                      bd_ = bd if bd in self.belo else bdr
+                      for belo_ in self.belo[bd_]:
+                          r_,Di_l,Di_u,Dj_l,Dj_u,be_l = belo_
+                          fl = tf.where( tf.logical_and(tf.greater_equal(self.rbd[bd],r_),
+                                             tf.logical_and(
+                                                 tf.logical_and(tf.less_equal(self.Dbi[bd],Di_u),
+                                                                tf.greater_equal(self.Dbi[bd],Di_l)),
+                                                 tf.logical_and(tf.less_equal(self.Dbj[bd],Dj_u),
+                                                                tf.greater_equal(self.Dbj[bd],Dj_l))
+                                                         ) ), 1.0,0.0) ##### 
+                          # fl = tf.where(tf.greater_equal(self.rbd[bd],r_),1.0,0.0) ##### 
                           pen_e   = tf.reduce_sum(input_tensor=tf.nn.relu((be_l - self.EBD[bd])*fl))
-                       self.penalty_be[bd] = self.penalty_be[bd] + pen_e
-                # if not self.beup and not self.belo:
-                #    fe  = tf.where(tf.less_equal(self.rbd[bd],self.rc_bo[bd]),1.0,0.0) ##### r< r_e that bo > bore_
-                #    fluct_l = 1.0-self.fluctuation
-                #    pen_e = tf.reduce_sum(input_tensor=tf.nn.relu((self.bop_pi[bd]*fluct_l-self.esi[bd])*fe))
-                #    self.penalty_be[bd] = self.penalty_be[bd] + pen_e
+                          self.penalty_be[bd] = self.penalty_be[bd] + pen_e
 
-                #    fluct_u = 1.0+self.fluctuation
-                #    pen_e = tf.reduce_sum(input_tensor=tf.nn.relu((self.esi[bd]-self.bop_pi[bd]*fluct_u)*fe))
-                #    self.penalty_be[bd] = self.penalty_be[bd] + pen_e
                 penalty = tf.add(self.penalty_be[bd]*self.lambda_bd,penalty) 
 
              # if self.MessageFunction==3:
@@ -1286,3 +1261,48 @@ class MPNN(ReaxFF):
              print('Vdw fluctuation penalty of                        {:5s}: {:6.4f}'.format(bd,penalty_vdw[bd]))
       print('\n')
       # self.log_energies()
+
+  def plot_result(self,step,E,dfte):
+      if not exists('results'):
+         makedirs('results')
+      Y,Yp = [],[]
+      for mol in self.mols:
+          maxe = self.max_e[mol]
+          plt.figure()
+          plt.ylabel('Energies comparation between DFT and ReaxFF')
+          plt.xlabel('Step')
+          #err  = dfte[mol] - E[mol]
+          Y.extend(dfte[mol]-maxe)
+          Yp.extend(E[mol]-maxe)
+
+          plt.plot(self.lx,dfte[mol]-maxe,linestyle='-',marker='o',markerfacecolor='snow',
+                   markeredgewidth=1,markeredgecolor='k',
+                   ms=5,c='k',alpha=0.8,label=r'$DFT$')
+          plt.plot(E[mol]-maxe,linestyle='-',marker='^',markerfacecolor='snow',
+                   markeredgewidth=1,markeredgecolor='b',
+                   ms=5,c='b',alpha=0.8,label=r'$I-ReaxFF$')
+          # plt.errorbar(self.lx,E[mol]-maxe,yerr=err,
+          #              fmt='-s',ecolor='r',color='r',ms=4,markerfacecolor='none',mec='blue',
+          #              elinewidth=2,capsize=2,label='I-ReaxFF')
+
+          plt.legend(loc='best',edgecolor='yellowgreen')
+          if step is None:
+             plt.savefig('results/result_%s.pdf' %mol) 
+          else:
+             plt.savefig('results/result_%s_%s.pdf' %(mol,step)) # transparent=True
+          plt.close()
+
+      plt.figure()
+      plt.xlabel('E(DFT)')
+      plt.ylabel('E(ReaxFF-nn)')
+      plt.scatter(Y,Yp,
+                  marker='o',color='none',edgecolor='r',s=20,
+                  alpha=0.8,label=r'$E(DFT) V.S. E(ReaxFF-nn)$')
+      plt.savefig('results/Result.svg')
+      plt.close()
+
+      with open('results/Results.csv','w') as fcsv:
+           print('Edft,Epred',file=fcsv)
+           for y,yp in zip(Y,Yp):
+               print(y,yp,sep=',',file=fcsv)
+
