@@ -200,13 +200,14 @@ class reax_data(object):
       self.max_e     = max(self.energy_nw)
 
       self.natom_images = self.natom                                    # all atoms include images
-      r_  = 2.0* max(self.r_cut.values())
+      r_  = max(self.r_cut.values()) # * 2.0
       R_  = np.sqrt(np.sum(self.cell*self.cell,axis=1))
 
       self.image_mask = [0,0,0]
       for i in range(3):
           if R_[i]<r_:
              self.image_mask[i] = 1
+             
       self.R,self.vr,self.R_image,self.vr_image = self.compute_bond(self.x,self.natom,self.nframe)
       image_rs = self.compute_image(self.vr)                            # vdw interaction images
 
@@ -357,8 +358,7 @@ class reax_data(object):
       self.bond = []
       bf        = []
       self.max_nei = 0
-
-      bondpair = []
+      # bondpair = []
       for i in range(self.natom): 
           if len(self.table[i])>self.max_nei:
              self.max_nei = len(self.table[i])
